if($pref::Server::RTP::RoundLimit $= "")
{
   $pref::Server::RTP::RoundLimit = 3;
}

$SK::Initialized = false;

function SK_BuildTowerList()
{
   //
   %pattern = "Add-Ons/RTP_*/save.bls";
   
   $SK::numTowers = 0;
   
   %file = findFirstFile(%pattern);
   while(%file !$= "")
   {
      $SK::Tower[$SK::numTowers] = %file;
      $SK::numTowers++;

      %file = findNextFile(%pattern);
   }
}

function SK_DumpTowerList()
{
   echo("");
   if($SK::numTowers == 1)
      echo("1 Tower");
   else
      echo($SK::numTowers @ " Towers");
   for(%i = 0; %i < $SK::numTowers; %i++)
   {
      %displayName = $SK::Tower[%i];
      %displayName = strReplace(%displayName, "Add-Ons/RTP_", "");
      %displayName = strReplace(%displayName, "/save.bls", "");
      %displayName = strReplace(%displayName, "_", " ");

      if(%i == $SK::CurrentTower)
         echo(" >" @ %displayName);
      else
         echo("  " @ %displayName);
   }
   echo("");
}

function SK_NextTower()
{
   $SK::CurrentTower = mFloor($SK::CurrentTower);
   $SK::CurrentTower++;
   $SK::CurrentTower = $SK::CurrentTower % $SK::numTowers;

   $SK::ResetCount = 0;

   SK_LoadTower_Phase1($SK::Tower[$SK::CurrentTower]);
}

function SK_LoadTower_Phase1(%filename)
{
   //suspend minigame resets
   $SK::MapChange = 1;

   //put everyone in observer mode
   %mg = $DefaultMiniGame;
   if(!isObject(%mg))
   {
      error("ERROR: SK_LoadTower( " @ %filename  @ " ) - default minigame does not exist");
      return;
   }
   for(%i = 0; %i < %mg.numMembers; %i++)
   {
      %client = %mg.member[%i];
      %player = %client.player;
      if(isObject(%player))
         %player.delete();

      %camera = %client.camera;
      %camera.setFlyMode();
      %camera.mode = "Observer";
      %client.setControlObject(%camera);
   }
   
   //clear all bricks 
   // note: this function is deferred, so we'll have to set a callback to be triggered when it's done
   BrickGroup_888888.chaindeletecallback = "SK_LoadTower_Phase2(\"" @ %filename @ "\");";
	BrickGroup_888888.chaindeleteall();
}

function SK_LoadTower_Phase2(%filename)
{
   echo("Loading RTP Tower " @ %filename);

   %displayName = %filename;
   %displayName = strReplace(%displayName, "Add-Ons/RTP_", "");
   %displayName = strReplace(%displayName, "/save.bls", "");
   %displayName = strReplace(%displayName, "_", " ");
   
   %loadMsg = "\c5Now loading \c6" @ %displayName;

   //read and display credits file, if it exists
   // limited to one line
   %creditsFilename = filePath(%fileName) @ "/credits.txt";
   if(isFile(%creditsFilename))
   {
      %file = new FileObject();
      %file.openforRead(%creditsFilename);

      %line = %file.readLine();
      %line = stripMLControlChars(%line);
      %loadMsg = %loadMsg @ "\c5, created by \c3" @ %line;

      %file.close();
      %file.delete();
   }

   messageAll('', %loadMsg);

   //load environment if it exists
   %envFile = filePath(%fileName) @ "/environment.txt"; 
   if(isFile(%envFile))
   {  
      //echo("parsing env file " @ %envFile);
      //usage: GameModeGuiServer::ParseGameModeFile(%filename, %append);
      //if %append == 0, all minigame variables will be cleared 
      %res = GameModeGuiServer::ParseGameModeFile(%envFile, 1);

      EnvGuiServer::getIdxFromFilenames();
      EnvGuiServer::SetSimpleMode();

      if(!$EnvGuiServer::SimpleMode)     
      {
         EnvGuiServer::fillAdvancedVarsFromSimple();
         EnvGuiServer::SetAdvancedMode();
      }
   }
   
   //load save file
   schedule(10, 0, serverDirectSaveFileLoad, %fileName, 3, "", 2, 1);
}


//some horrible /commands to change Towers and such
function serverCmdTowerList(%client)
{
   for(%i = 0; %i < $SK::numTowers; %i++)
   {
      %displayName = $SK::Tower[%i];
      %displayName = strReplace(%displayName, "Add-Ons/RTP_", "");
      %displayName = strReplace(%displayName, "/save.bls", "");
      %displayName = strReplace(%displayName, "_", " ");

      if(%i == $SK::CurrentTower)
         messageClient(%client, '', " >" @ %i @ ". \c6" @ %displayName);
      else
         messageClient(%client, '', "  " @ %i @ ". \c6" @ %displayName);
   }
}
function serverCmdSetTower(%client, %i)
{
   if(!%client.isAdmin)
      return;

   if(mFloor(%i) !$= %i)
   {
      messageClient(%client, '', "Usage: /setTower <number>");
      return;
   }

   if(%i < 0 || %i > $SK::numTowers)
   {
      messageClient(%client, '', "serverCmdSetTower() - out of range index");
      return;
   }

   messageAll( 'MsgAdminForce', '\c3%1\c2 changed the Tower', %client.getPlayerName());
   
   $SK::CurrentTower = %i - 1;
   SK_NextTower();
}

function serverCmdNextTower(%client, %i)
{
   if(!%client.isAdmin)
      return;

   messageAll( 'MsgAdminForce', '\c3%1\c2 changed the Tower', %client.getPlayerName());
   
   SK_NextTower();
}

package GameModeRTPPackage
{
   //this is called when save loading finishes 
   function GameModeInitialResetCheck()
   {
      Parent::GameModeInitialResetCheck();

      //if there is no Tower list, attempt to create it
      if($SK::numTowers == 0)
         SK_BuildTowerList();
      
      //if Towerlist is still empty, there are no Towers
      if($SK::numTowers == 0)
      {
         messageAll('', "\c5No RTP Towers available!");
         return;
      }

      if($SK::Initialized)
         return;

      $SK::Initialized = true;
      $SK::CurrentTower = -1;
            
      SK_NextTower();
   }

   //when we're done loading a new Tower, reset the minigame
   function ServerLoadSaveFile_End()
   {
      Parent::ServerLoadSaveFile_End();

      //new Tower has loaded, reset minigame
      if($DefaultMiniGame.numMembers > 0) //don't bother if no one is here (this also prevents starting at round 2 on server creation)
         $DefaultMiniGame.scheduleReset(); //don't do it instantly, to give people a little bit of time to ghost
   }
   

   function MiniGameSO::Reset(%obj, %client)
   {
      //make sure this value is an number
      $pref::Server::RTP::RoundLimit = mFloor($pref::Server::RTP::RoundLimit);

      //handle our race time hack
      %obj.raceStartTime = -1500;

      //count number of minigame resets, when we reach the limit, go to next Tower
      if(%obj.numMembers >= 0)
      {
         $SK::ResetCount++;
      }

      if($SK::ResetCount > $pref::Server::RTP::RoundLimit)
      {
         $SK::ResetCount = 0;
         SK_NextTower();
      }
      else
      {
         messageAll('', "\c5Beginning round " @ $SK::ResetCount @ " of " @ $pref::Server::RTP::RoundLimit);
         Parent::Reset(%obj, %client);
      }
   }  
};
activatePackage(GameModeRTPPackage);

// more time to fly out of a burning car
$CorpseTimeoutValue = 7000;

//special event to explode vehicles that are left in the garage
registerOutputEvent("fxDTSBrick", "explodeNearVehicle", "");
function fxDTSBrick::explodeNearVehicle(%obj)
{
   %vehicle = %obj.vehicle;
   if(!isObject(%vehicle))
      return;

   %delta = vectorSub(%vehicle.getPosition(), %obj.getPosition());
   //echo("len = " @ vectorLen(%delta));
   if(vectorLen(%delta) < 5) //7.5)
      %vehicle.finalExplosion(); //damage(%vehicle, %vehicle.getPosition(), 10000, $DamageType::Default);
}


//special event to win the race, displays race time
registerOutputEvent("GameConnection", "winRace", "");
function GameConnection::winRace(%client)
{
   %mg = %client.miniGame;

   if(!isObject(%mg))
      %mg = $DefaultMiniGame;

   if(!isObject(%mg))
      return;

   //if race start time is not available, use time since last reset
   %startTime = %mg.raceStartTime;
   if(%startTime <= 0)
      %startTime = %mg.lastResetTime;

   %elapsedTime = getSimTime() - %startTime - 15000;
   %elapsedTime = mFloor(%elapsedTime / 1000);
   
   %mg.chatMessageAll(0,  "\c3" @ %client.getPlayerName() @ " \c5RESCUED THE PRINCESS IN \c6" @ getTimeString(%elapsedTime) @ "\c3!");
   %mg.scheduleReset(7000);
}
